; Below is a shell program created from:
;    Andrew Davies' 2600 Programming Tutorials on Atari Age,
;    The Stella Mailing List,
;    and Atari Age's 2600 Programming Forum(s) in general.

; It includes:
; * Basic Setup (processor, includes, output of scanlines so it'll compile as-is.
; * Magic 9-byte Zeropage/TIA zeroing procedure.
; * Variable Overlay Definition framework.
; * Timer-Based Vertical Retrace & Overscan Sections.
; * Horizontal Sprite Positioning Routine (Why reinvent the wheel?)

;----------------------------------------------------------------------
; <Project Title>
; by <programmer(s)>
; console: Atari 2600

  processor 6502
  include "vcs.h"
  include "macro.h"

; Overlay RAM Setup.
  SEG.U RAM_Variables ; the label "RAM_Variables" will appear in our symbol table's segment list
  ORG $80             ; start of RAM

; This overlay variable is used for the overlay variables.
; FOR SAFETY, DO NOT USE THIS AREA DIRECTLY (ie: NEVER reference
; 'Overlay' in the code) ADD AN OVERLAY FOR EACH ROUTINE'S USE,
; SO CLASHES CAN BE EASILY CHECKED.

Overlay ds 0 ; Set this 0 to the size of the largest overlay defined
             ; below in the Overlay sections.

;--------------------------------------------------------------------------
; OVERLAYS! (Information)
; These variables are overlays, and should be managed with care.
; That is, variables are ALREADY DEFINED, and we're reusing RAM
; for other purposes.  These things are essentially local variables.
; You can add as many overlays as required, so long as they are mutually
; exclusive. 

; TOTAL SPACE USED BY ANY OVERLAY GROUP SHOULD BE <= SIZE OF 'Overlay'

;---------------------------------------------------------------
    
  org Overlay
  ; Generic Name for programmer, ie: ANIMATION/LOGIC SYSTEM
  ; place variables here

;---------------------------------------------------------------

  org Overlay
  ; DRAWING SYSTEM/Etc. 
  ; place variables here


;---------------(Variable Overlays Completed)-------------------

  SEG
  ORG $F000

Reset


; Start of Brilliantly Obscure Inilitization Procedure by Andrew Davies.
; CLEARS ALL VARIABLES, STACK
; INIT STACK POINTER
; ALSO CLEARS TIA REGISTERS
; DOES THIS BY "WRAPPING" THE STACK - UNUSUAL

  ldx #0
  txa
Clear dex
  txs
  pha
  bne Clear

; 8 BYTES TOTAL FOR CLEARING STACK, MEMORY
; STACK POINTER NOW $FF, A=X==0
; End of Brilliantly Obscure Initiliazation Procedure by Andrew Davies.  



StartOfFrame

; Start of vertical blank processing

  lda #0
  sta VBLANK

  lda #2
  sta VSYNC
           
; 3 scanlines of VSYNCH signal...

  sta WSYNC
  sta WSYNC
  sta WSYNC

  lda #0
  sta VSYNC           
      
        
; Entering Timer-based Display Screen Vertical Blank logic Segment.  
  lda #43       ; Vblank for 37 lines
  sta TIM64T    ; 43*64intvls=2752=8256colclks=36.2lines
; ------------- Enter Vertical Blank Logic Below -------------------

;		Add your Vertical Blank code Here.  If your game's display
;       starts rolling after adding code here, you are probably taking
;       up too much time and thus generating too many scanlines!

; ------------- Enter Vertical Blank Logic Above -------------------
DisplayScreenVBlankLoop
  lda INTIM
  bne DisplayScreenVBlankLoop   ; wait for vblank timer
  sta WSYNC		                ; finish waiting for the current line


; 192 scanlines of picture...

  ldx #0
  REPEAT 192 ; scanlines
    inx
    stx COLUBK
    sta WSYNC
  REPEND

  
              
  lda #%01000010
  sta VBLANK       ; end of screen - enter blanking
        
; Entering Timer-based Display Screen OverScan logic Segment.  
  lda #35       ;skip 30 lines (overscan)
  sta TIM64T
; ------------- Enter OverScan Logic Below -------------------

;		Add your OverScan code here.  If your game's display
;       starts rolling after adding code here, you are probably taking
;       up too much time and thus generating too many scanlines!

; ------------- Enter Overscan Logic Above -------------------
DisplayOverScanLoop
  lda INTIM
  bne DisplayOverScanLoop    ; wait for Overscan timer
  sta WSYNC                  ; finish waiting for the current line

  
  jmp StartOfFrame


; Positioning Code via Atari Age Forums which likely originated from the Stella Mailing List.    
; I've seen this procedure come up several times.
PositionASpriteSubroutine
  sta HMCLR
  sec
  sta WSYNC         ;                       begin line 1
DivideLoop
  sbc #15
  bcs DivideLoop    ;+4/5    4/ 9.../54

  eor #7            ;+2      6/11.../56
  asl
  asl
  asl
  asl               ;+8     14/19.../64

  sta.wx HMP0,X     ;+5     19/24.../69
  sta RESP0,X       ;+4     23/28/33/38/43/48/53/58/63/68/73
  sta WSYNC         ;+3      0              begin line 2
  sta HMOVE         ;+3
  rts               ;+6      9
   
; Set X to the index into which object you want to position 
; 	(0 = player 0, 1 = player 1, 2 = missile 0, 3 = missile 1, 4 = ball)

; Set A to the pixel you want to position the object at. The nice thing about 
; this routine is that, for normal-width players, setting A to zero will position 
; the player at the left-most edge of the screen. Setting A to 159 will position 
; the player at the right-most edge of the screen.

; The other nice thing about this routine is that it will always always take 
; exactly the same number of cycles* - call this with 14 cycles left in the 
; scanline and it will always return 99 cycles later (9 cycles into the 2nd scanline).

; *If you feed it a position between 0 and 159
  
  
  ORG $FFFA

  .word Reset ; NMI
  .word Reset ; RESET
  .word Reset ; IRQ

  END
